/************************************************************************
*  File: de5_router_pcie_diag.c
*
*  Sample user-mode diagnostics application for accessing DE5_ROUTER_PCIE
*  devices using WinDriver's API.
*  Code was generated by DriverWizard v11.5.0.
*
*  Jungo Connectivity Confidential. Copyright (c) 2011 Jungo Connectivity Ltd.  http://www.jungo.com
*************************************************************************/

#include <stdio.h>
#include "wdc_defs.h"
#include "wdc_lib.h"
#include "utils.h"
#include "status_strings.h"
#include "samples/shared/diag_lib.h"
#include "samples/shared/wdc_diag_lib.h"
#include "pci_regs.h"
#include "de5_router_pcie_lib.h"
#include "router_tables.h"

/*************************************************************
  General definitions
 *************************************************************/
/* Error messages display */
#define DE5_ROUTER_PCIE_ERR printf

/*************************************************************
  Global variables
 *************************************************************/
/* User's input command */
static CHAR gsInput[256];

/* --------------------------------------------------
    DE5_ROUTER_PCIE configuration registers information
   -------------------------------------------------- */
/* Configuration registers information array */
const WDC_REG gDE5_ROUTER_PCIE_CfgRegs[] = {
    { WDC_AD_CFG_SPACE, PCI_VID, WDC_SIZE_16, WDC_READ_WRITE, "VID", "Vendor ID" },
    { WDC_AD_CFG_SPACE, PCI_DID, WDC_SIZE_16, WDC_READ_WRITE, "DID", "Device ID" },
    { WDC_AD_CFG_SPACE, PCI_CR, WDC_SIZE_16, WDC_READ_WRITE, "CMD", "Command" },
    { WDC_AD_CFG_SPACE, PCI_SR, WDC_SIZE_16, WDC_READ_WRITE, "STS", "Status" },
    { WDC_AD_CFG_SPACE, PCI_REV, WDC_SIZE_32, WDC_READ_WRITE, "RID_CLCD", "Revision ID & Class Code" },
    { WDC_AD_CFG_SPACE, PCI_CCSC, WDC_SIZE_8, WDC_READ_WRITE, "SCC", "Sub Class Code" },
    { WDC_AD_CFG_SPACE, PCI_CCBC, WDC_SIZE_8, WDC_READ_WRITE, "BCC", "Base Class Code" },
    { WDC_AD_CFG_SPACE, PCI_CLSR, WDC_SIZE_8, WDC_READ_WRITE, "CALN", "Cache Line Size" },
    { WDC_AD_CFG_SPACE, PCI_LTR, WDC_SIZE_8, WDC_READ_WRITE, "LAT", "Latency Timer" },
    { WDC_AD_CFG_SPACE, PCI_HDR, WDC_SIZE_8, WDC_READ_WRITE, "HDR", "Header Type" },
    { WDC_AD_CFG_SPACE, PCI_BISTR, WDC_SIZE_8, WDC_READ_WRITE, "BIST", "Built-in Self Test" },
    { WDC_AD_CFG_SPACE, PCI_BAR0, WDC_SIZE_32, WDC_READ_WRITE, "BADDR0", "Base Address 0" },
    { WDC_AD_CFG_SPACE, PCI_BAR1, WDC_SIZE_32, WDC_READ_WRITE, "BADDR1", "Base Address 1" },
    { WDC_AD_CFG_SPACE, PCI_BAR2, WDC_SIZE_32, WDC_READ_WRITE, "BADDR2", "Base Address 2" },
    { WDC_AD_CFG_SPACE, PCI_BAR3, WDC_SIZE_32, WDC_READ_WRITE, "BADDR3", "Base Address 3" },
    { WDC_AD_CFG_SPACE, PCI_BAR4, WDC_SIZE_32, WDC_READ_WRITE, "BADDR4", "Base Address 4" },
    { WDC_AD_CFG_SPACE, PCI_BAR5, WDC_SIZE_32, WDC_READ_WRITE, "BADDR5", "Base Address 5" },
    { WDC_AD_CFG_SPACE, PCI_CIS, WDC_SIZE_32, WDC_READ_WRITE, "CIS", "CardBus CIS Pointer" },
    { WDC_AD_CFG_SPACE, PCI_SVID, WDC_SIZE_16, WDC_READ_WRITE, "SVID", "Sub-system Vendor ID" },
    { WDC_AD_CFG_SPACE, PCI_SDID, WDC_SIZE_16, WDC_READ_WRITE, "SDID", "Sub-system Device ID" },
    { WDC_AD_CFG_SPACE, PCI_EROM, WDC_SIZE_32, WDC_READ_WRITE, "EROM", "Expansion ROM Base Address" },
    { WDC_AD_CFG_SPACE, PCI_CAP, WDC_SIZE_8, WDC_READ_WRITE, "NEW_CAP", "New Capabilities Pointer" },
    { WDC_AD_CFG_SPACE, PCI_ILR, WDC_SIZE_32, WDC_READ_WRITE, "INTLN", "Interrupt Line" },
    { WDC_AD_CFG_SPACE, PCI_IPR, WDC_SIZE_32, WDC_READ_WRITE, "INTPIN", "Interrupt Pin" },
    { WDC_AD_CFG_SPACE, PCI_MGR, WDC_SIZE_32, WDC_READ_WRITE, "MINGNT", "Minimum Required Burst Period" },
    { WDC_AD_CFG_SPACE, PCI_MLR, WDC_SIZE_32, WDC_READ_WRITE, "MAXLAT", "Maximum Latency" },
    };
#define DE5_ROUTER_PCIE_CFG_REGS_NUM sizeof(gDE5_ROUTER_PCIE_CfgRegs) / sizeof(WDC_REG)
/* TODO: For read-only or write-only registers, change the direction field of
         the relevant registers in gDE5_ROUTER_PCIE_CfgRegs to WDC_READ or WDC_WRITE. */
/* NOTE: You can define additional configuration registers in gDE5_ROUTER_PCIE_CfgRegs. */
const WDC_REG *gpDE5_ROUTER_PCIE_CfgRegs = gDE5_ROUTER_PCIE_CfgRegs;

/* -----------------------------------------------
    DE5_ROUTER_PCIE run-time registers information
   ----------------------------------------------- */
/* Run-time registers information array */
/* const WDC_REG gDE5_ROUTER_PCIE_Regs[]; */
const WDC_REG *gpDE5_ROUTER_PCIE_Regs = NULL;
/* TODO: You can remove the comment from the gDE5_ROUTER_PCIE_Regs array declaration and
         fill the array with run-time registers information for your device,
         in which case be sure to set gpDE5_ROUTER_PCIE_Regs to point to gDE5_ROUTER_PCIE_Regs. */
#define DE5_ROUTER_PCIE_REGS_NUM 0

/*************************************************************
  Static functions prototypes
 *************************************************************/
/* -----------------------------------------------
    Main diagnostics menu
   ----------------------------------------------- */
static void MenuMain(WDC_DEVICE_HANDLE *phDev);

/* -----------------------------------------------
    Device find, open and close
   ----------------------------------------------- */
static WDC_DEVICE_HANDLE DeviceFindAndOpen(DWORD dwVendorId, DWORD dwDeviceId);
static BOOL DeviceFind(DWORD dwVendorId, DWORD dwDeviceId, WD_PCI_SLOT *pSlot);
static WDC_DEVICE_HANDLE DeviceOpen(const WD_PCI_SLOT *pSlot);
static void DeviceClose(WDC_DEVICE_HANDLE hDev);

/* -----------------------------------------------
    Read/write memory and I/O addresses
   ----------------------------------------------- */
static void MenuReadWriteAddr(WDC_DEVICE_HANDLE hDev);
static void SetAddrSpace(WDC_DEVICE_HANDLE hDev, PDWORD pdwAddrSpace);

/* -----------------------------------------------
    Read/write the configuration space
   ----------------------------------------------- */
static void MenuReadWriteCfgSpace(WDC_DEVICE_HANDLE hDev);

/* -----------------------------------------------
    Read/write the run-time registers
   ----------------------------------------------- */
static void MenuReadWriteRegs(WDC_DEVICE_HANDLE hDev);

/* -----------------------------------------------
    Interrupt handling
   ----------------------------------------------- */
static void MenuInterrupts(WDC_DEVICE_HANDLE hDev);
static void DiagIntHandler(WDC_DEVICE_HANDLE hDev, DE5_ROUTER_PCIE_INT_RESULT *pIntResult);

/* ----------------------------------------------------
    Plug-and-play and power management events handling
   ---------------------------------------------------- */
static void MenuEvents(WDC_DEVICE_HANDLE hDev);
static void DiagEventHandler(WDC_DEVICE_HANDLE hDev, DWORD dwAction);

unsigned int OnRCSlaveRead(WDC_DEVICE_HANDLE hDev, int bar, unsigned int addr);
void OnRCSlaveWrite(WDC_DEVICE_HANDLE hDev, int bar, unsigned int addr, unsigned int u32Data);
void set_router_tables(WDC_DEVICE_HANDLE hDev);
void get_ip_addr(char *ip_address);
void get_mac_addr(char *mac_address);
void get_port(char *port_number);
unsigned int MAC_HI_ADDR;
unsigned int MAC_LO_ADDR;
unsigned int IP_ADDR;
unsigned int PORT_NUM;

/*************************************************************
  Functions implementation
 *************************************************************/
int main(void)
{
    WDC_DEVICE_HANDLE hDev = NULL;
    DWORD dwStatus;

    printf("\n");
    printf("DE5_ROUTER_PCIE diagnostic utility.\n");
    printf("Application accesses hardware using " WD_PROD_NAME ".\n");

    /* Initialize the DE5_ROUTER_PCIE library */
    dwStatus = DE5_ROUTER_PCIE_LibInit();
    if (WD_STATUS_SUCCESS != dwStatus)
    {
        DE5_ROUTER_PCIE_ERR("de5_router_pcie_diag: Failed to initialize the DE5_ROUTER_PCIE library: %s",
            DE5_ROUTER_PCIE_GetLastErr());
        return dwStatus;
    }

    /* Find and open a DE5_ROUTER_PCIE device (by default ID) */
    if (DE5_ROUTER_PCIE_DEFAULT_VENDOR_ID)
        hDev = DeviceFindAndOpen(DE5_ROUTER_PCIE_DEFAULT_VENDOR_ID, DE5_ROUTER_PCIE_DEFAULT_DEVICE_ID);

    /* Display main diagnostics menu for communicating with the device */
    //MenuMain(&hDev);

	set_router_tables(hDev);

    /* Perform necessary cleanup before exiting the program */
    if (hDev)
        DeviceClose(hDev);
 
    dwStatus = DE5_ROUTER_PCIE_LibUninit();
    if (WD_STATUS_SUCCESS != dwStatus)
        DE5_ROUTER_PCIE_ERR("de5_router_pcie_diag: Failed to uninit the DE5_ROUTER_PCIE library: %s", DE5_ROUTER_PCIE_GetLastErr());
    
    return dwStatus;
}

/* -----------------------------------------------
    Main diagnostics menu
   ----------------------------------------------- */
/* Main menu options */
enum {
    MENU_MAIN_SCAN_PCI_BUS = 1,
    MENU_MAIN_FIND_AND_OPEN,
    MENU_MAIN_RW_ADDR,
    MENU_MAIN_RW_CFG_SPACE,
    MENU_MAIN_RW_REGS,
    MENU_MAIN_ENABLE_DISABLE_INT,
    MENU_MAIN_EVENTS,
    MENU_MAIN_EXIT = DIAG_EXIT_MENU,
};

/* Main diagnostics menu */
static void MenuMain(WDC_DEVICE_HANDLE *phDev)
{
    DWORD option;
    
    do
    {
        printf("\n");
        printf("DE5_ROUTER_PCIE main menu\n");
        printf("--------------\n");
        printf("%d. Scan PCI bus\n", MENU_MAIN_SCAN_PCI_BUS);
        printf("%d. Find and open a DE5_ROUTER_PCIE device\n", MENU_MAIN_FIND_AND_OPEN);
        if (*phDev)
        {
            printf("%d. Read/write memory and IO addresses on the device\n",
                MENU_MAIN_RW_ADDR);
            printf("%d. Read/write the device's configuration space\n",
                MENU_MAIN_RW_CFG_SPACE);
            if (DE5_ROUTER_PCIE_REGS_NUM)
            {
                printf("%d. Read/write the run-time registers\n",
                    MENU_MAIN_RW_REGS);
            }
            printf("%d. Enable/disable the device's interrupts\n",
                MENU_MAIN_ENABLE_DISABLE_INT);
            printf("%d. Register/unregister plug-and-play and power management "
                "events\n", MENU_MAIN_EVENTS);
        }
        printf("%d. Exit\n", MENU_MAIN_EXIT);
        
        if (DIAG_INPUT_FAIL == DIAG_GetMenuOption(&option,
            *phDev ? MENU_MAIN_EVENTS : MENU_MAIN_FIND_AND_OPEN))
        {
            continue;
        }

        switch (option)
        {
        case MENU_MAIN_EXIT: /* Exit menu */
            break;
        case MENU_MAIN_SCAN_PCI_BUS: /* Scan PCI bus */
            WDC_DIAG_PciDevicesInfoPrintAll(FALSE);
            break;
        case MENU_MAIN_FIND_AND_OPEN: /* Find and open a DE5_ROUTER_PCIE device */
            if (*phDev)
                DeviceClose(*phDev);
            *phDev = DeviceFindAndOpen(0, 0);
            break;
        case MENU_MAIN_RW_ADDR: /* Read/write memory and I/O addresses */
            MenuReadWriteAddr(*phDev);
            break;
        case MENU_MAIN_RW_CFG_SPACE: /* Read/Write the DE5_ROUTER_PCIE configuration space */
            MenuReadWriteCfgSpace(*phDev);
            break;
        case MENU_MAIN_RW_REGS: /* Read/write the run-time registers */
            if (DE5_ROUTER_PCIE_REGS_NUM)
                MenuReadWriteRegs(*phDev);
            else
                printf("Invalid selection\n");
            break;
        case MENU_MAIN_ENABLE_DISABLE_INT: /* Enable/disable interrupts */
            MenuInterrupts(*phDev);
            break;
        case MENU_MAIN_EVENTS: /* Register/unregister plug-and-play and power management events */
            MenuEvents(*phDev);
            break;
        }
    } while (MENU_MAIN_EXIT != option);
}

/* -----------------------------------------------
    Device find, open and close
   ----------------------------------------------- */
/* Find and open a DE5_ROUTER_PCIE device */
static WDC_DEVICE_HANDLE DeviceFindAndOpen(DWORD dwVendorId, DWORD dwDeviceId)
{
    WD_PCI_SLOT slot;
    
    if (!DeviceFind(dwVendorId, dwDeviceId, &slot))
        return NULL;

    return DeviceOpen(&slot);
}

/* Find a DE5_ROUTER_PCIE device */
static BOOL DeviceFind(DWORD dwVendorId, DWORD dwDeviceId, WD_PCI_SLOT *pSlot)
{
    DWORD dwStatus;
    DWORD i, dwNumDevices;
    WDC_PCI_SCAN_RESULT scanResult;

    if (dwVendorId == 0)
    {
        if (DIAG_INPUT_SUCCESS != DIAG_InputDWORD((PVOID)&dwVendorId,
            "Enter vendor ID", TRUE, 0, 0))
        {
            return FALSE;
        }

        if (DIAG_INPUT_SUCCESS != DIAG_InputDWORD((PVOID)&dwDeviceId,
            "Enter device ID", TRUE, 0, 0))
        {
            return FALSE;
        }
    }

    BZERO(scanResult);
    dwStatus = WDC_PciScanDevices(dwVendorId, dwDeviceId, &scanResult);
    if (WD_STATUS_SUCCESS != dwStatus)
    {
        DE5_ROUTER_PCIE_ERR("DeviceFind: Failed scanning the PCI bus.\n"
            "Error: 0x%lx - %s\n", dwStatus, Stat2Str(dwStatus));
        return FALSE;
    }

    dwNumDevices = scanResult.dwNumDevices;
    if (!dwNumDevices)
    {
        DE5_ROUTER_PCIE_ERR("No matching device was found for search criteria "
            "(Vendor ID 0x%lX, Device ID 0x%lX)\n",
            dwVendorId, dwDeviceId);

        return FALSE;
    }
    
    printf("\n");
    printf("Found %ld matching device%s [ Vendor ID 0x%lX%s, Device ID 0x%lX%s ]:\n",
        dwNumDevices, dwNumDevices > 1 ? "s" : "",
        dwVendorId, dwVendorId ? "" : " (ALL)",
        dwDeviceId, dwDeviceId ? "" : " (ALL)");
    
    for (i = 0; i < dwNumDevices; i++)
    {
        printf("\n");
        printf("%2ld. Vendor ID: 0x%lX, Device ID: 0x%lX\n",
            i + 1,
            scanResult.deviceId[i].dwVendorId,
            scanResult.deviceId[i].dwDeviceId);

        WDC_DIAG_PciDeviceInfoPrint(&scanResult.deviceSlot[i], FALSE);
    }
    printf("\n");

    if (dwNumDevices > 1)
    {
        sprintf(gsInput, "Select a device (1 - %ld): ", dwNumDevices);
        i = 0;
        if (DIAG_INPUT_SUCCESS != DIAG_InputDWORD((PVOID)&i,
            gsInput, FALSE, 1, dwNumDevices))
        {
            return FALSE;
        }
    }

    *pSlot = scanResult.deviceSlot[i - 1];

    return TRUE;
}

/* Open a handle to a DE5_ROUTER_PCIE device */
static WDC_DEVICE_HANDLE DeviceOpen(const WD_PCI_SLOT *pSlot)
{
    WDC_DEVICE_HANDLE hDev;
    DWORD dwStatus;
    WD_PCI_CARD_INFO deviceInfo;
    
    /* Retrieve the device's resources information */
    BZERO(deviceInfo);
    deviceInfo.pciSlot = *pSlot;
    dwStatus = WDC_PciGetDeviceInfo(&deviceInfo);
    if (WD_STATUS_SUCCESS != dwStatus)
    {
        DE5_ROUTER_PCIE_ERR("DeviceOpen: Failed retrieving the device's resources information.\n"
            "Error 0x%lx - %s\n", dwStatus, Stat2Str(dwStatus));
        return NULL;
    }

    /* NOTE: You can modify the device's resources information here, if
       necessary (mainly the deviceInfo.Card.Items array or the items number -
       deviceInfo.Card.dwItems) in order to register only some of the resources
       or register only a portion of a specific address space, for example. */

    /* Open a handle to the device */
    hDev = DE5_ROUTER_PCIE_DeviceOpen(&deviceInfo);
    if (!hDev)
    {
        DE5_ROUTER_PCIE_ERR("DeviceOpen: Failed opening a handle to the device: %s",
            DE5_ROUTER_PCIE_GetLastErr());
        return NULL;
    }

    return hDev;
}

/* Close handle to a DE5_ROUTER_PCIE device */
static void DeviceClose(WDC_DEVICE_HANDLE hDev)
{
    if (!hDev)
        return;

    if (!DE5_ROUTER_PCIE_DeviceClose(hDev))
    {
        DE5_ROUTER_PCIE_ERR("DeviceClose: Failed closing DE5_ROUTER_PCIE device: %s",
            DE5_ROUTER_PCIE_GetLastErr());
    }
}

/* -----------------------------------------------
    Read/write memory and I/O addresses
   ----------------------------------------------- */
/* Read/write address menu options */
enum {
    MENU_RW_ADDR_SET_ADDR_SPACE = 1,
    MENU_RW_ADDR_SET_MODE,
    MENU_RW_ADDR_SET_TRANS_TYPE,
    MENU_RW_ADDR_READ,
    MENU_RW_ADDR_WRITE,
    MENU_RW_ADDR_EXIT = DIAG_EXIT_MENU,
};

#define ACTIVE_ADDR_SPACE_NEEDS_INIT 0xFF

/* Read/write memory or I/O space address menu */
static void MenuReadWriteAddr(WDC_DEVICE_HANDLE hDev)
{
    DWORD option;
    static DWORD dwAddrSpace = ACTIVE_ADDR_SPACE_NEEDS_INIT;
    static WDC_ADDR_MODE mode = WDC_MODE_32;
    static BOOL fBlock = FALSE;

    /* Initialize active address space */
    if (ACTIVE_ADDR_SPACE_NEEDS_INIT == dwAddrSpace)
    {
        DWORD dwNumAddrSpaces = DE5_ROUTER_PCIE_GetNumAddrSpaces(hDev);
        
        /* Find the first active address space */
        for (dwAddrSpace = 0; dwAddrSpace < dwNumAddrSpaces; dwAddrSpace++)
        {
            if (WDC_AddrSpaceIsActive(hDev, dwAddrSpace))
                break;
        }
        
        /* Sanity check */
        if (dwAddrSpace == dwNumAddrSpaces)
        {
            DE5_ROUTER_PCIE_ERR("MenuReadWriteAddr: Error - no active address spaces found\n");
            dwAddrSpace = ACTIVE_ADDR_SPACE_NEEDS_INIT;
            return;
        }
    }

    do
    {
        printf("\n");
        printf("Read/write the device's memory and IO ranges\n");
        printf("---------------------------------------------\n");
        printf("%d. Change active address space for read/write "
            "(currently: BAR %ld)\n", MENU_RW_ADDR_SET_ADDR_SPACE, dwAddrSpace);
        printf("%d. Change active read/write mode (currently: %s)\n",
            MENU_RW_ADDR_SET_MODE,
            (WDC_MODE_8 == mode) ? "8 bit" : (WDC_MODE_16 == mode) ? "16 bit" :
            (WDC_MODE_32 == mode) ? "32 bit" : "64 bit");
        printf("%d. Toggle active transfer type (currently: %s)\n",
            MENU_RW_ADDR_SET_TRANS_TYPE,
            (fBlock ? "block transfers" : "non-block transfers"));
        printf("%d. Read from active address space\n", MENU_RW_ADDR_READ);
        printf("%d. Write to active address space\n", MENU_RW_ADDR_WRITE);
        printf("%d. Exit menu\n", MENU_RW_ADDR_EXIT);
        printf("\n");
        
        if (DIAG_INPUT_FAIL == DIAG_GetMenuOption(&option,
            MENU_RW_ADDR_WRITE))
        {
            continue;
        }
        
        switch (option)
        {
        case MENU_RW_ADDR_EXIT: /* Exit menu */
            break;
        case MENU_RW_ADDR_SET_ADDR_SPACE: /* Set active address space for read/write address requests */
        {
            SetAddrSpace(hDev, &dwAddrSpace);
            break;
        }
        case MENU_RW_ADDR_SET_MODE: /* Set active mode for read/write address requests */
            WDC_DIAG_SetMode(&mode);
            break;
        case MENU_RW_ADDR_SET_TRANS_TYPE: /* Toggle active transfer type */
            fBlock = !fBlock;
            break;
        case MENU_RW_ADDR_READ:  /* Read from a memory or I/O address */
        case MENU_RW_ADDR_WRITE: /* Write to a memory or I/O address */
        {
            WDC_DIRECTION direction =
                (MENU_RW_ADDR_READ == option) ? WDC_READ : WDC_WRITE;

            if (fBlock)
                WDC_DIAG_ReadWriteBlock(hDev, direction, dwAddrSpace);
            else
                WDC_DIAG_ReadWriteAddr(hDev, direction, dwAddrSpace, mode);
            
            break;
        }
        }
    } while (MENU_RW_ADDR_EXIT != option);
}

static void SetAddrSpace(WDC_DEVICE_HANDLE hDev, PDWORD pdwAddrSpace)
{
    DWORD dwAddrSpace;
    DWORD dwNumAddrSpaces = DE5_ROUTER_PCIE_GetNumAddrSpaces(hDev);
    DE5_ROUTER_PCIE_ADDR_SPACE_INFO addrSpaceInfo;
    
    printf("\n");
    printf("Select an active address space:\n");
    printf("-------------------------------\n");

    for (dwAddrSpace = 0; dwAddrSpace < dwNumAddrSpaces; dwAddrSpace++)
    {
        BZERO(addrSpaceInfo);
        addrSpaceInfo.dwAddrSpace = dwAddrSpace;
        if (!DE5_ROUTER_PCIE_GetAddrSpaceInfo(hDev, &addrSpaceInfo))
        {
            DE5_ROUTER_PCIE_ERR("SetAddrSpace: Error - Failed to get address space information: %s",
                DE5_ROUTER_PCIE_GetLastErr());
            return;
        }

        printf("%ld. %-*s %-*s %s\n",
            dwAddrSpace + 1,
            MAX_NAME_DISPLAY, addrSpaceInfo.sName,
            MAX_TYPE - 1, addrSpaceInfo.sType,
            addrSpaceInfo.sDesc);
    }
    printf("\n");

    if (DIAG_INPUT_SUCCESS != DIAG_InputDWORD((PVOID)&dwAddrSpace,
        "Enter option", FALSE, 1, dwNumAddrSpaces))
    {
         return;
    }

    dwAddrSpace--;
    if (!WDC_AddrSpaceIsActive(hDev, dwAddrSpace))
    {
        printf("You have selected an inactive address space\n");
        return;
    }
            
    *pdwAddrSpace = dwAddrSpace;
}

/* -----------------------------------------------
    Read/write the configuration space
   ----------------------------------------------- */
/* Read/write the configuration space menu options */
enum {
    MENU_RW_CFG_SPACE_READ_OFFSET = 1,
    MENU_RW_CFG_SPACE_WRITE_OFFSET,
    MENU_RW_CFG_SPACE_READ_ALL_REGS,
    MENU_RW_CFG_SPACE_READ_REG,
    MENU_RW_CFG_SPACE_WRITE_REG,
    MENU_RW_CFG_SPACE_EXIT = DIAG_EXIT_MENU,
};

/* Display read/write configuration space menu */
static void MenuReadWriteCfgSpace(WDC_DEVICE_HANDLE hDev)
{
    DWORD option;
  
    do {
        /* Display pre-defined registers' information */
        if (DE5_ROUTER_PCIE_CFG_REGS_NUM)
        {
            printf("\n");
            printf("Configuration registers:\n");
            printf("------------------------\n");
            WDC_DIAG_RegsInfoPrint(gpDE5_ROUTER_PCIE_CfgRegs, DE5_ROUTER_PCIE_CFG_REGS_NUM,
                WDC_DIAG_REG_PRINT_ALL & ~WDC_DIAG_REG_PRINT_ADDR_SPACE);
        }

        printf("\n");
        printf("Read/write the device's configuration space\n");
        printf("--------------------------------------------\n");
        printf("%d. Read from an offset\n", MENU_RW_CFG_SPACE_READ_OFFSET);
        printf("%d. Write to an offset\n", MENU_RW_CFG_SPACE_WRITE_OFFSET);
        if (DE5_ROUTER_PCIE_CFG_REGS_NUM)
        {
            printf("%d. Read all configuration registers defined for the device (see list above)\n",
                MENU_RW_CFG_SPACE_READ_ALL_REGS);
            printf("%d. Read from a named register\n", MENU_RW_CFG_SPACE_READ_REG);
            printf("%d. Write to a named register\n", MENU_RW_CFG_SPACE_WRITE_REG);
        }
        printf("%d. Exit menu\n", MENU_RW_CFG_SPACE_EXIT);
        printf("\n");

        if (DIAG_INPUT_FAIL == DIAG_GetMenuOption(&option,
            DE5_ROUTER_PCIE_CFG_REGS_NUM ? MENU_RW_CFG_SPACE_WRITE_REG :
            MENU_RW_CFG_SPACE_WRITE_OFFSET))
        {
            continue;
        }
        
        switch (option)
        {
        case MENU_RW_CFG_SPACE_EXIT: /* Exit menu */
            break;
        case MENU_RW_CFG_SPACE_READ_OFFSET: /* Read from a configuration space offset */
            WDC_DIAG_ReadWriteBlock(hDev, WDC_READ, WDC_AD_CFG_SPACE);
            break;
        case MENU_RW_CFG_SPACE_WRITE_OFFSET: /* Write to a configuration space offset */
            WDC_DIAG_ReadWriteBlock(hDev, WDC_WRITE, WDC_AD_CFG_SPACE);
            break;
        case MENU_RW_CFG_SPACE_READ_ALL_REGS:
            WDC_DIAG_ReadRegsAll(hDev, gpDE5_ROUTER_PCIE_CfgRegs, DE5_ROUTER_PCIE_CFG_REGS_NUM, TRUE);
            break;
        case MENU_RW_CFG_SPACE_READ_REG:  /* Read from a configuration register */
            WDC_DIAG_ReadWriteReg(hDev, gpDE5_ROUTER_PCIE_CfgRegs, DE5_ROUTER_PCIE_CFG_REGS_NUM, WDC_READ, TRUE);
            break;
        case MENU_RW_CFG_SPACE_WRITE_REG: /* Write to a configuration register */
            WDC_DIAG_ReadWriteReg(hDev, gpDE5_ROUTER_PCIE_CfgRegs, DE5_ROUTER_PCIE_CFG_REGS_NUM, WDC_WRITE, TRUE);
            break;
        }
    } while (MENU_RW_CFG_SPACE_EXIT != option);
}

/* -----------------------------------------------
    Read/write the run-time registers
   ----------------------------------------------- */
/* Read/write the run-time registers menu options */
enum {
    MENU_RW_REGS_READ_ALL = 1,
    MENU_RW_REGS_READ_REG,
    MENU_RW_REGS_WRITE_REG,
    MENU_RW_REGS_EXIT = DIAG_EXIT_MENU,
};

/* Display read/write run-time registers menu */
static void MenuReadWriteRegs(WDC_DEVICE_HANDLE hDev)
{
    DWORD option;
    
    if (!DE5_ROUTER_PCIE_REGS_NUM)
    {
        printf("There are currently no pre-defined run-time registers\n");
        return;
    }
  
    do {
        /* Display pre-defined registers' information */
        printf("\n");
        printf("DE5_ROUTER_PCIE run-time registers:\n");
        printf("--------------------------\n");   
        WDC_DIAG_RegsInfoPrint(gpDE5_ROUTER_PCIE_Regs, DE5_ROUTER_PCIE_REGS_NUM, WDC_DIAG_REG_PRINT_ALL);

        printf("\n");
        printf("Read/write the DE5_ROUTER_PCIE run-time registers\n");
        printf("-----------------------------------------\n");
        printf("%d. Read all run-time registers defined for the device (see list above)\n",
            MENU_RW_REGS_READ_ALL);
        printf("%d. Read from a specific register\n", MENU_RW_REGS_READ_REG);
        printf("%d. Write to a specific register\n", MENU_RW_REGS_WRITE_REG);
        printf("%d. Exit menu\n", MENU_RW_REGS_EXIT);
        printf("\n");

        if (DIAG_INPUT_FAIL == DIAG_GetMenuOption(&option,
            MENU_RW_REGS_WRITE_REG))
        {
            continue;
        }

        switch (option)
        {
        case MENU_RW_REGS_EXIT: /* Exit menu */
            break;
        case MENU_RW_REGS_READ_ALL:
            WDC_DIAG_ReadRegsAll(hDev, gpDE5_ROUTER_PCIE_Regs, DE5_ROUTER_PCIE_REGS_NUM, FALSE);
            break;
        case MENU_RW_REGS_READ_REG:  /* Read from a register */
            WDC_DIAG_ReadWriteReg(hDev, gpDE5_ROUTER_PCIE_Regs, DE5_ROUTER_PCIE_REGS_NUM, WDC_READ, FALSE);
            break;
        case MENU_RW_REGS_WRITE_REG: /* Write to a register */
            WDC_DIAG_ReadWriteReg(hDev, gpDE5_ROUTER_PCIE_Regs, DE5_ROUTER_PCIE_REGS_NUM, WDC_WRITE, FALSE);
            break;
        }
    } while (MENU_RW_REGS_EXIT != option);
}

/* -----------------------------------------------
    Interrupt handling
   ----------------------------------------------- */
/* Interrupts menu options */
enum {
    MENU_INT_ENABLE_DISABLE = 1,
    MENU_INT_EXIT = DIAG_EXIT_MENU,
};

/* Enable/Disable interrupts menu */
static void MenuInterrupts(WDC_DEVICE_HANDLE hDev)
{
    DWORD option, dwIntOptions;
    BOOL fIntEnable, fIsMsi;

    dwIntOptions = WDC_GET_INT_OPTIONS(hDev);
    fIsMsi = WDC_INT_IS_MSI(dwIntOptions);
    if (dwIntOptions & INTERRUPT_LEVEL_SENSITIVE)
    {
        /* TODO: You can remove this message after you have modified the
           implementation of DE5_ROUTER_PCIE_IntEnable() in de5_router_pcie_lib.c to 
           correctly acknowledge level-sensitive interrupts (see guidelines
           in DE5_ROUTER_PCIE_IntEnable()) */
        printf("\n");
        printf("WARNING!!!\n");
        printf("----------\n");
        printf("Your hardware has level sensitive interrupts.\n");
        printf("Before enabling the interrupts, %s first modify the source "
           "code of DE5_ROUTER_PCIE_IntEnable(), in the file de5_router_pcie_lib.c, to "
           "correctly acknowledge\n%s interrupts when they occur (as dictated by "
           "the hardware's specifications)\n",
           fIsMsi ? "it's recommended that you" : "you must",
           fIsMsi ? "level sensitive" : "");    }

    do
    {
        fIntEnable = !DE5_ROUTER_PCIE_IntIsEnabled(hDev);

        printf("\n");
        printf("Interrupts\n");
        printf("-----------\n");
        printf("%d. %s interrupts\n", MENU_INT_ENABLE_DISABLE,
            fIntEnable ? "Enable" : "Disable");
        printf("%d. Exit menu\n", MENU_INT_EXIT);
        printf("\n");

        if (DIAG_INPUT_FAIL == DIAG_GetMenuOption(&option,
            MENU_RW_ADDR_WRITE))
        {
            continue;
        }

        switch (option)
        {
        case MENU_INT_EXIT: /* Exit menu */
            break;
        case MENU_INT_ENABLE_DISABLE: /* Enable/disable interrupts */
            if (fIntEnable)
            {
                DWORD dwStatus = DE5_ROUTER_PCIE_IntEnable(hDev, DiagIntHandler);

                if (WD_STATUS_SUCCESS == dwStatus)
                    printf("Interrupts enabled\n");
                else
                {
                    DE5_ROUTER_PCIE_ERR("Failed enabling interrupts. Error 0x%lx - %s\n",
                       dwStatus, Stat2Str(dwStatus));
                }
            }
            else
            {
                if (WD_STATUS_SUCCESS == DE5_ROUTER_PCIE_IntDisable(hDev))
                    printf("Interrupts disabled\n");
                else
                    DE5_ROUTER_PCIE_ERR("Failed disabling interrupts: %s", DE5_ROUTER_PCIE_GetLastErr());
            }
            break;
        }
    } while (MENU_INT_EXIT != option);
}

/* Diagnostics interrupt handler routine */
static void DiagIntHandler(WDC_DEVICE_HANDLE hDev, DE5_ROUTER_PCIE_INT_RESULT *pIntResult)
{
    /* TODO: You can modify this function in order to implement your own
             diagnostics interrupt handler routine */

    printf("Got interrupt number %ld\n", pIntResult->dwCounter);
    printf("Interrupt Type: %s\n",
        WDC_DIAG_IntTypeDescriptionGet(pIntResult->dwEnabledIntType));
    if (WDC_INT_IS_MSI(pIntResult->dwEnabledIntType))
        printf("Message Data: 0x%lx\n", pIntResult->dwLastMessage);
}

/* ----------------------------------------------------
    Plug-and-play and power management events handling
   ---------------------------------------------------- */
/* Events menu options */
enum {
    MENU_EVENTS_REGISTER_UNREGISTER = 1,
    MENU_EVENTS_EXIT = DIAG_EXIT_MENU,
};

/* Register/unregister Plug-and-play and power management events */
static void MenuEvents(WDC_DEVICE_HANDLE hDev)
{
    DWORD option;
    BOOL fRegister;

    do
    {
        fRegister = !DE5_ROUTER_PCIE_EventIsRegistered(hDev);
        
        printf("\n");
        printf("Plug-and-play and power management events\n");
        printf("------------------------------------------\n");
        printf("%d. %s events\n", MENU_EVENTS_REGISTER_UNREGISTER,
            fRegister ? "Register" : "Unregister");
        printf("%d. Exit menu\n", MENU_EVENTS_EXIT);
        printf("\n");
        
        if (DIAG_INPUT_FAIL == DIAG_GetMenuOption(&option,
            MENU_EVENTS_REGISTER_UNREGISTER))
        {
            continue;
        }

        switch (option)
        {
        case MENU_EVENTS_EXIT: /* Exit menu */
            break;
        case MENU_EVENTS_REGISTER_UNREGISTER: /* Register/unregister events */
            if (fRegister)
            {
                if (WD_STATUS_SUCCESS == DE5_ROUTER_PCIE_EventRegister(hDev, DiagEventHandler))
                    printf("Events registered\n");
                else
                    DE5_ROUTER_PCIE_ERR("Failed to register events. Last error:\n%s", DE5_ROUTER_PCIE_GetLastErr());
            }
            else
            {
                if (WD_STATUS_SUCCESS == DE5_ROUTER_PCIE_EventUnregister(hDev))
                    printf("Events unregistered\n");
                else
                    DE5_ROUTER_PCIE_ERR("Failed to unregister events. Last Error:\n%s", DE5_ROUTER_PCIE_GetLastErr());
            }
            break;
        }
    } while (MENU_EVENTS_EXIT != option);
}

/* Plug-and-play and power management events handler routine */
static void DiagEventHandler(WDC_DEVICE_HANDLE hDev, DWORD dwAction)
{
    /* TODO: You can modify this function in order to implement your own
             diagnostics events handler routine */

    printf("\nReceived event notification (device handle 0x%p): ", hDev);
    switch (dwAction)
    {
    case WD_INSERT:
        printf("WD_INSERT\n");
        break;
    case WD_REMOVE:
        printf("WD_REMOVE\n");
        break;
    case WD_POWER_CHANGED_D0:
        printf("WD_POWER_CHANGED_D0\n");
        break;
    case WD_POWER_CHANGED_D1:
        printf("WD_POWER_CHANGED_D1\n");
        break;
    case WD_POWER_CHANGED_D2:
        printf("WD_POWER_CHANGED_D2\n");
        break;
    case WD_POWER_CHANGED_D3:
        printf("WD_POWER_CHANGED_D3\n");
        break;
    case WD_POWER_SYSTEM_WORKING:
        printf("WD_POWER_SYSTEM_WORKING\n");
        break;
    case WD_POWER_SYSTEM_SLEEPING1:
        printf("WD_POWER_SYSTEM_SLEEPING1\n");
        break;
    case WD_POWER_SYSTEM_SLEEPING2:
        printf("WD_POWER_SYSTEM_SLEEPING2\n");
        break;
    case WD_POWER_SYSTEM_SLEEPING3:
        printf("WD_POWER_SYSTEM_SLEEPING3\n");
        break;
    case WD_POWER_SYSTEM_HIBERNATE:
        printf("WD_POWER_SYSTEM_HIBERNATE\n");
        break;
    case WD_POWER_SYSTEM_SHUTDOWN:
        printf("WD_POWER_SYSTEM_SHUTDOWN\n");
        break;
    default:
        printf("0x%lx\n", dwAction);
        break;
    }
}

unsigned int OnRCSlaveRead(WDC_DEVICE_HANDLE hDev, int bar, unsigned int addr)
{
	unsigned int u32Data=0;
	WDC_ReadAddr32(hDev, bar, addr, &u32Data);
	return u32Data;
}

void OnRCSlaveWrite(WDC_DEVICE_HANDLE hDev, int bar, unsigned int addr, unsigned int u32Data)
{
   WDC_WriteAddr32(hDev,bar,addr,u32Data);
}

void get_mac_addr(char *mac_address)
{
	/* This function parses MAC address values
	*/
	unsigned int mac_addr[6];
	char str[30];
	strcpy(str,mac_address);
	sscanf((strtok(str,":")),"%x",&mac_addr[0]);
	sscanf((strtok(NULL,":")),"%x",&mac_addr[1]);
	sscanf((strtok(NULL,":")),"%x",&mac_addr[2]);
	sscanf((strtok(NULL,":")),"%x",&mac_addr[3]);
	sscanf((strtok(NULL,":")),"%x",&mac_addr[4]);
	sscanf((strtok(NULL,":")),"%x",&mac_addr[5]);
	MAC_HI_ADDR = (mac_addr[0]);
	MAC_HI_ADDR = (MAC_HI_ADDR << 8) + mac_addr[1];
	MAC_LO_ADDR = (mac_addr[2]);
	MAC_LO_ADDR = (MAC_LO_ADDR <<8 ) + mac_addr[3];
	MAC_LO_ADDR = (MAC_LO_ADDR <<8 ) + mac_addr[4];
	MAC_LO_ADDR = (MAC_LO_ADDR <<8 ) + mac_addr[5];
}

void get_ip_addr(char *ip_address)
{
	/* This function parses IP address values
	*/
	unsigned int ip_addr[4];
	char str[30];
	char buf[2];
	strcpy(str,ip_address);
	sscanf((strtok(str,".")),"%d",&ip_addr[0]);
	sscanf((strtok(NULL,".")),"%d",&ip_addr[1]);
	sscanf((strtok(NULL,".")),"%d",&ip_addr[2]);
	sscanf((strtok(NULL,".")),"%d",&ip_addr[3]);
	IP_ADDR = ip_addr[0];
	IP_ADDR = (IP_ADDR << 8) + ip_addr[1];
	IP_ADDR = (IP_ADDR << 8) + ip_addr[2];
	IP_ADDR = (IP_ADDR << 8) + ip_addr[3];

}

void get_port(char *port_number)
{
	/* This function parses Port number.
	*/
	unsigned int port_no;
	char str[5];
	strcpy(str,port_number);
	sscanf((strtok(str,"\0")),"%x",&port_no);
	PORT_NUM = port_no;
}

void set_router_tables(WDC_DEVICE_HANDLE hDev)
{
	/* This function configure the reference router routing table
	*/
	unsigned int ret;
	char *mac_address;
	char *ip_address;
	char *port_number;
	// MAC variables
	unsigned int MAC_0_HI;
	unsigned int MAC_0_LO;
	unsigned int MAC_1_HI;
	unsigned int MAC_1_LO;
	unsigned int MAC_2_HI;
	unsigned int MAC_2_LO;
	unsigned int MAC_3_HI;
	unsigned int MAC_3_LO;

	// DESTIP variables
	unsigned int IP_REG_00;
	unsigned int IP_REG_01;
	unsigned int IP_REG_02;
	unsigned int IP_REG_03;

	// ARP variables
	unsigned int ARP_NEXT_HOP_IP_REG_01; 
	unsigned int ARP_MAC_HI_REG_01; 
	unsigned int ARP_MAC_LO_REG_01; 
	unsigned int ARP_NEXT_HOP_IP_REG_02; 
	unsigned int ARP_MAC_HI_REG_02; 
	unsigned int ARP_MAC_LO_REG_02; 
	unsigned int ARP_NEXT_HOP_IP_REG_03; 
	unsigned int ARP_MAC_HI_REG_03; 
	unsigned int ARP_MAC_LO_REG_03; 
	unsigned int ARP_NEXT_HOP_IP_REG_04; 
	unsigned int ARP_MAC_HI_REG_04; 
	unsigned int ARP_MAC_LO_REG_04; 

	// LPM variables
	unsigned int LPM_IP_REG_01; 
	unsigned int MASK_REG_01;
	unsigned int LPM_NEXT_HOP_IP_REG_01; 
	unsigned int LPM_OUTPUT_PORT_REG_01;
	unsigned int LPM_IP_REG_02; 
	unsigned int MASK_REG_02;
	unsigned int LPM_NEXT_HOP_IP_REG_02; 
	unsigned int LPM_OUTPUT_PORT_REG_02;
	unsigned int LPM_IP_REG_03; 
	unsigned int MASK_REG_03;
	unsigned int LPM_NEXT_HOP_IP_REG_03; 
	unsigned int LPM_OUTPUT_PORT_REG_03;
	unsigned int LPM_IP_REG_04; 
	unsigned long MASK_REG_04;
	unsigned int LPM_NEXT_HOP_IP_REG_04; 
	unsigned int LPM_OUTPUT_PORT_REG_04;

	// MAC 0
	mac_address = MAC_ADDR_0;
	get_mac_addr(mac_address);
	MAC_0_HI = MAC_HI_ADDR;
	MAC_0_LO = MAC_LO_ADDR;

	// MAC 1
	mac_address = MAC_ADDR_1;
	get_mac_addr(mac_address);
	MAC_1_HI = MAC_HI_ADDR;
	MAC_1_LO = MAC_LO_ADDR;

	// MAC 2
	mac_address = MAC_ADDR_2;
	get_mac_addr(mac_address);
	MAC_2_HI = MAC_HI_ADDR;
	MAC_2_LO = MAC_LO_ADDR;

	// MAC 3
	mac_address = MAC_ADDR_3;
	get_mac_addr(mac_address);
	MAC_3_HI = MAC_HI_ADDR;
	MAC_3_LO = MAC_LO_ADDR;

	// Destip 0
	ip_address = DEST_IP_0;
	get_ip_addr(ip_address);
	IP_REG_00 = IP_ADDR;

	// Destip 1
	ip_address = DEST_IP_1;
	get_ip_addr(ip_address);
	IP_REG_01 = IP_ADDR;

	// Destip 2
	ip_address = DEST_IP_2;
	get_ip_addr(ip_address);
	IP_REG_02 = IP_ADDR;

	// Destip 3
	ip_address = DEST_IP_3;
	get_ip_addr(ip_address);
	IP_REG_03 = IP_ADDR;

	// lpm 1
	ip_address = LPM_IP_1;
	get_ip_addr(ip_address);
	LPM_IP_REG_01 = IP_ADDR;

	ip_address = LPM_MASK_1;
	get_ip_addr(ip_address);
	MASK_REG_01 = IP_ADDR;

	ip_address = LPM_NEXT_HOP_IP_1;
	get_ip_addr(ip_address);
	LPM_NEXT_HOP_IP_REG_01 = IP_ADDR;

	port_number = LPM_OUTPUT_PORT_1;
	get_port(port_number);
	LPM_OUTPUT_PORT_REG_01 = PORT_NUM;

	// lpm 2
	ip_address = LPM_IP_2;
	get_ip_addr(ip_address);
	LPM_IP_REG_02 = IP_ADDR;

	ip_address = LPM_MASK_2;
	get_ip_addr(ip_address);
	MASK_REG_02 = IP_ADDR;

	ip_address = LPM_NEXT_HOP_IP_2;
	get_ip_addr(ip_address);
	LPM_NEXT_HOP_IP_REG_02 = IP_ADDR;

	port_number = LPM_OUTPUT_PORT_2;
	get_port(port_number);
	LPM_OUTPUT_PORT_REG_02 = PORT_NUM;

	// Arp 1
	ip_address = NEXT_HOP_IP_1;
	get_ip_addr(ip_address);
	ARP_NEXT_HOP_IP_REG_01 = IP_ADDR;

	mac_address = NEXT_HOP_MAC_1;
	get_mac_addr(mac_address);
	ARP_MAC_HI_REG_01 = MAC_HI_ADDR;
	ARP_MAC_LO_REG_01 = MAC_LO_ADDR;

	// Arp 2
	ip_address = NEXT_HOP_IP_2;
	get_ip_addr(ip_address);
	ARP_NEXT_HOP_IP_REG_02 = IP_ADDR;

	mac_address = NEXT_HOP_MAC_2;
	get_mac_addr(mac_address);
	ARP_MAC_HI_REG_02 = MAC_HI_ADDR;
	ARP_MAC_LO_REG_02 = MAC_LO_ADDR;

	// Arp 3
	ip_address = NEXT_HOP_IP_3;
	get_ip_addr(ip_address);
	ARP_NEXT_HOP_IP_REG_03 = IP_ADDR;

	mac_address = NEXT_HOP_MAC_3;
	get_mac_addr(mac_address);
	ARP_MAC_HI_REG_03 = MAC_HI_ADDR;
	ARP_MAC_LO_REG_03 = MAC_LO_ADDR;

	// Arp 4
	ip_address = NEXT_HOP_IP_4;
	get_ip_addr(ip_address);
	ARP_NEXT_HOP_IP_REG_04 = IP_ADDR;

	mac_address = NEXT_HOP_MAC_4;
	get_mac_addr(mac_address);
	ARP_MAC_HI_REG_04 = MAC_HI_ADDR;
	ARP_MAC_LO_REG_04 = MAC_LO_ADDR;

   /********************  MAC Table **********************/	
    
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_0_HI_REG, MAC_0_HI);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_0_LO_REG, MAC_0_LO);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_1_HI_REG, MAC_1_HI);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_1_LO_REG, MAC_1_LO);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_2_HI_REG, MAC_2_HI);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_2_LO_REG, MAC_2_LO);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_3_HI_REG, MAC_3_HI);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_MAC_3_LO_REG, MAC_3_LO);
	
	
   /********************  DST_IP Table **********************/	
     
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_ENTRY_IP_REG, IP_REG_00);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_WR_ADDR_REG, WR_ADDR_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_ENTRY_IP_REG, IP_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_WR_ADDR_REG, WR_ADDR_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_ENTRY_IP_REG, IP_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_WR_ADDR_REG, WR_ADDR_REG_03);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_ENTRY_IP_REG, IP_REG_03);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_DST_IP_FILTER_TABLE_WR_ADDR_REG, WR_ADDR_REG_04);
	
	/********************  ARP Table **********************/	
	   
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_NEXT_HOP_IP_REG, ARP_NEXT_HOP_IP_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_HI_REG, ARP_MAC_HI_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_LO_REG, ARP_MAC_LO_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_WR_ADDR_REG, ARP_WR_ADDR_REG_01);

	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_NEXT_HOP_IP_REG, ARP_NEXT_HOP_IP_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_HI_REG, ARP_MAC_HI_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_LO_REG, ARP_MAC_LO_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_WR_ADDR_REG, ARP_WR_ADDR_REG_02);
	
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_NEXT_HOP_IP_REG, ARP_NEXT_HOP_IP_REG_03);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_HI_REG, ARP_MAC_HI_REG_03);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_LO_REG, ARP_MAC_LO_REG_03);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_WR_ADDR_REG, ARP_WR_ADDR_REG_03);
	
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_NEXT_HOP_IP_REG, ARP_NEXT_HOP_IP_REG_04);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_HI_REG, ARP_MAC_HI_REG_04);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_ENTRY_MAC_LO_REG, ARP_MAC_LO_REG_04);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ARP_TABLE_WR_ADDR_REG, ARP_WR_ADDR_REG_04);
	
	/********************  LPM Table **********************/	

	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_IP_REG, LPM_IP_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_MASK_REG, MASK_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_NEXT_HOP_IP_REG, LPM_NEXT_HOP_IP_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_OUTPUT_PORT_REG, LPM_OUTPUT_PORT_REG_02);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_WR_ADDR_REG, LPM_WR_ADDR_REG_02);	
	Sleep(1000);// Add a small delay between multiple LPM writes. To make sure CAM is written properly
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_IP_REG, LPM_IP_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_MASK_REG, MASK_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_NEXT_HOP_IP_REG, LPM_NEXT_HOP_IP_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_ENTRY_OUTPUT_PORT_REG, LPM_OUTPUT_PORT_REG_01);
	OnRCSlaveWrite(hDev, 0, ROUTER_OP_LUT_ROUTE_TABLE_WR_ADDR_REG, LPM_WR_ADDR_REG_01);


	
	printf("Router Table Configuration ends \n");
}
